#!/usr/bin/env python3
"""
ELF Write 1
by Javantea
Dec 14, 2014

Create an ELF file from scratch. We've got quite a ways to go.
./out4
Segmentation fault
objdump -x out4
objdump: out4: File truncated
hexdump -C out4 
00000000  7f 45 4c 46 02 01 01 00  00 00 00 00 00 00 00 00  |.ELF............|
00000010  02 00 3e 00 01 00 00 00  9c 47 40 00 00 00 00 00  |..>......G@.....|
00000020  40 00 00 00 00 00 00 00  70 02 00 00 00 00 00 00  |@.......p.......|
00000030  00 00 00 00 40 00 38 00  01 00 40 00 01 00 1a 00  |....@.8...@.....|
00000040  06 00 00 00 05 00 00 00  40 00 00 00 00 00 00 00  |........@.......|
00000050  40 00 40 00 00 00 00 00  40 00 40 00 00 00 00 00  |@.@.....@.@.....|
00000060  30 02 00 00 00 00 00 00  30 02 00 00 00 00 00 00  |0.......0.......|
00000070  08 00 00 00 00 00 00 00  00 00 00 00 00 00 00 00  |................|
00000080  00 00 00 00 00 00 00 00  00 00 00 00 00 00 00 00  |................|
*
000000b0  00 00 00 00 00 00 00 00                           |........|
000000b8

"""
from __future__ import print_function
import elf1

def main():
	import sys
	import os
	
	filename = 'out'
	if len(sys.argv) > 1:
		filename = sys.argv[1]
	#end if
	
	if os.path.exists(filename):
		print("Error: Unwilling to overwite \"%s\"." % filename)
		sys.exit(1)
	#end if
	
	e = elf1.elf_header(b'', error='64')

	# 64-bit
	e.e_ident = b'\x7fELF\x02\x01\x01\x00\x00\x00\x00\x00\x00\x00\x00\x00'
	e.e_type = 2
	e.e_machine = 62
	e.e_version = 1
	e.e_entry = 0x40479c
	e.e_phoff = 64
	e.e_shoff = e.e_phoff + (e.e_phentsize * e.e_phnum)
	e.e_flags = 0
	e.e_ehsize = 64
	e.e_phentsize = 56
	e.e_phnum = 1 # len(proghs)
	e.e_shentsize = 64
	e.e_shnum = 1 # len(sectionhs)
	e.e_shstrndx = 26

	proghs = []
	sectionhs = []
	
	ph = elf1.prog_header(b'', e, 0, error='default')
	ph.p_type = 6
	ph.p_flags = 5
	ph.p_offset = 64
	ph.p_vaddr = 0x400040
	ph.p_paddr = 0x400040
	ph.p_filesz = 560
	ph.p_memsz = 560
	ph.p_align = 8
	proghs.append(ph)

	sh = elf1.section_header(b'', e, 0, error='default')
	sh.sh_name = 0
	sh.sh_type = 0
	sh.sh_flags = 0
	sh.sh_addr = 0
	sh.sh_offset = 0
	sh.sh_size = 0
	sh.sh_link = 0
	sh.sh_info = 0
	sh.sh_addralign = 0
	sh.sh_entsize = 0
	sectionhs.append(sh)
	
	f = open(filename, 'wb')
	
	f.write(e.out())
	for progh in proghs:
		f.write(progh.out())
	#next progh
	for sectionh in sectionhs:
		f.write(sectionh.out())
	#next sectionh
	f.close()
#end def main()

if __name__ == '__main__':
	main()
#end if
